import os
from sklearn.feature_extraction import  DictVectorizer
from sklearn.naive_bayes import  GaussianNB, MultinomialNB
from sklearn.model_selection import train_test_split, GridSearchCV
from sklearn.tree import DecisionTreeClassifier
from sklearn.neighbors import  KNeighborsClassifier
import time

import sklearn.metrics as met
from termcolor import colored

import pandas as pd

def class_info(clf, x_train, y_train, x_test, y_test, cv=False):

    start=time.time()
    clf.fit(x_train, y_train)

    end=time.time()
    print('Vreme', end-start)

    if cv:
        print('Najbolji parametri', clf.best_params_)

        means = clf.cv_results_['mean_test_score']
        stds = clf.cv_results_['std_test_score']
        for mean, std, params in zip(means, stds, clf.cv_results_['params']):
            print("%0.3f (+/-%0.03f) za %s" % (mean, std * 2, params))

    print('Trening skup')
    y_pred = clf.predict(x_train)

    cnf_matrix = met.confusion_matrix(y_train, y_pred)
    print("Matrica konfuzije", pd.DataFrame(cnf_matrix, index=clf.classes_, columns=clf.classes_), sep="\n")
    print("\n")

    accuracy = met.accuracy_score(y_train, y_pred)
    print("Preciznost", accuracy)
    print("\n")

    class_report = met.classification_report(y_train, y_pred, target_names=clf.classes_)
    print("Izvestaj klasifikacije", class_report, sep="\n")


    print('Test skup')

    y_pred = clf.predict(x_test)

    cnf_matrix = met.confusion_matrix(y_test, y_pred)
    print("Matrica konfuzije", pd.DataFrame(cnf_matrix, index=clf.classes_, columns=clf.classes_), sep="\n")
    print("\n")

    accuracy = met.accuracy_score(y_test, y_pred)
    print("Preciznost", accuracy)
    print("\n")

    class_report = met.classification_report(y_test, y_pred, target_names=clf.classes_)
    print("Izvestaj klasifikacije", class_report, sep="\n")



corpus = []
classes = []
for class_name in os.listdir('Skup'):
    dir_path = 'Skup\\' + class_name
    for file_name in os.listdir(dir_path):
        f=open(dir_path + '\\' +file_name, 'r')
        file_words = {}
        for line in f:
            word_count=line.split()
            file_words[word_count[0]]=int(word_count[1])
        classes.append(class_name)
        #print(file_words)

        corpus.append(file_words)
        f.close()

dv = DictVectorizer()

x=dv.fit_transform(corpus)

print(dv.get_feature_names())

df=pd.DataFrame(x.toarray(), columns=dv.get_feature_names())
#df['class']=classes

#print(df['class'])


x_train, x_test, y_train, y_test = train_test_split(df, classes, train_size=0.7, stratify=classes)

print(colored("MultinomialNB", "blue"))

"""
MultinomialNB
parametri:
    alpha : parametar ugladjivanja, vrednosti [0,1]
          default=1
    fit_prior  : da li da uci verovatnoce klasa
           default = True
           vrednosti:
                    True
                    False - uzima se uniforma raspodela klasa
    class_prior: zadate verovatnoce klasa
"""

clf_mnb = MultinomialNB()
class_info(clf_mnb, x_train, y_train, x_test, y_test)

parameters = [{'criterion': ['gini', 'entropy'],
               'max_depth':[15, 25, 5, 50],
               }]

print(colored("DecisionTreeClassifier", "blue"))

clf_dt = GridSearchCV(DecisionTreeClassifier(), parameters, cv=5)
class_info(clf_dt, x_train, y_train, x_test, y_test, cv=True)


"""
print(colored("KNeighborsClassifier", "blue"))
parameters = [{'n_neighbors': range(3,10),
               'p':[1, 2],
               'weights': ['uniform', 'distance'],
               }]

clf_dt = GridSearchCV(KNeighborsClassifier(), parameters, cv=5 )
class_info(clf_dt, x_train, y_train, x_test, y_test, cv=True)


"""
